#include "include/udp_client.h"

#include <stdio.h>
#include "include/udp_payload.h"

udp_client_t udp_client;

static inline void handle_receive(struct pbuf *p, const ip_addr_t *addr, u16_t port)
{
    if(p->len >= 2)
    {
        uint8_t *payload = (uint8_t *)p->payload;
        uint16_t len = p->len;

        udp_client.message_callback(payload, len, addr, port);
    }

    pbuf_free(p);
}

static void udp_receive_callback(void *arg, struct udp_pcb *pcb, struct pbuf *p, const ip_addr_t *addr, u16_t port)
{
    udp_client_t *udp_client_received_data = (udp_client_t *)arg;
    handle_receive(p, addr, port);
}

// Default callback func
static void default_message_callback(uint8_t *payload, uint16_t len, const ip_addr_t *addr, uint16_t port)
{
    printf("Received: packet=%d, len=%d from %s:%d\n", payload[0], len, ipaddr_ntoa(addr), port);
}

void udp_client_init(void)
{
    udp_client.message_callback = false ? udp_client_message_handler : default_message_callback;

    udp_client.pcb = udp_new();
    if(udp_client.pcb == NULL)
    {
        puts("Error creating UDP client");
        return;
    }

    udp_recv(udp_client.pcb, udp_receive_callback, &udp_client);

    err_t err = udp_bind(udp_client.pcb, IP_ADDR_ANY, UDP_CLIENT_PORT);
    if(err != ERR_OK)
    {
        printf("Erreur bind UDP client: %d\n", err);
        return;
    }

    printf("UDP client started on port %d\n", UDP_CLIENT_PORT);
}

void udp_client_exit(void)
{
    if(udp_client.pcb)
    {
        udp_remove(udp_client.pcb);
        udp_client.pcb = NULL;
    }
}
