#include "include/wifi_operator.h"

#include <stdio.h>
#include "pico/cyw43_arch.h"
#include "lwip/netif.h"
#include "lwip/ip4_addr.h"

void wifi_operator_init(void)
{
    // Mode client
    cyw43_arch_enable_sta_mode();

    // Désactiver le mode d'économie d'énergie
    cyw43_wifi_pm(&cyw43_state, CYW43_NO_POWERSAVE_MODE);

    // Afficher les paramètres de connexion
    printf("Tentative de connexion à: SSID='%s'\n", WIFI_OPERATOR_SSID);

    // Configuration IP
    ip4_addr_t ip, netmask, gateway;
    IP4_ADDR(&ip, 192, 168, 128, 2);
    IP4_ADDR(&netmask, 255, 255, 255, 0);
    IP4_ADDR(&gateway, 192, 168, 128, 1);

    // Configuration réseau avant connexion
    netif_set_up(netif_default);
    netif_set_addr(netif_default, &ip, &netmask, &gateway);

    puts("Configuration IP effectuée");

    int connect_result = 1;
    // Tentative de connexion
    puts("Attente avant connexion...");
    sleep_ms(2000);  // Attendre 2 secondes que le WiFi soit bien initialisé

    do
    {
        serialDebug("Tentative de connexion au réseau Wi-Fi...");
        connect_result = cyw43_arch_wifi_connect_timeout_ms(ssid, password, CYW43_AUTH_WPA2_AES_PSK, 10000);

        if(connect_result != 0)
        {
            char error_message[100];
            snprintf(error_message, sizeof(error_message), "Échec de la connexion WiFi - Code d'erreur: %d", connect_result);
            serialDebug(error_message);

            const char *error_description;
            switch (connect_result)
            {
                case -1:
                error_description = "Erreur générale";
                break;
                case -2:
                error_description = "Point d'accès non trouvé";
                break;
                case -3:
                error_description = "Mot de passe incorrect";
                break;
                default:
                error_description = "Erreur inconnue";
            }
            puts(error_description);
        }
    }
    while(connect_result);

    // Configuration de l'interface réseau
    if(!netif_default)
    {
        puts("Erreur: Interface réseau non disponible");
        return;
    }

    netif_set_up(netif_default);
    netif_set_link_up(netif_default);
    netif_set_addr(netif_default, &ip, &netmask, &gateway);

    puts("Connexion établie avec succès!");
}
